<?php

/*
Plugin Name:  Pannos Preview Links
Plugin URI:   https://www.pannosmarketing.com/
Description:  A simple plugin to enable a limited time preview link for non-users to view draft pages.
Version:      1.3
Author:       Max Hann | Pannos Marketing
Author URI:   https://www.pannosmarketing.com
License:      GPL2
License URI:  https://www.gnu.org/licenses/gpl-2.0.html
Text Domain:  preview-links
Domain Path:  /languages
*/


use Carbon_Fields\Container;
use Carbon_Fields\Block;
use Carbon_Fields\Field;

add_action( 'carbon_fields_register_fields', 'preview_link_option' );
function preview_link_option() {


    if ( function_exists('carbon_get_post_meta') ) {

    /* Custom Settings Fields */

        Container::make( 'theme_options', __( 'Pannos Preview Link Options' ) )
            ->add_fields( array(
                Field::make( 'text', 'pm_draft_preview_types', __( 'Post Types Allowing Preview Links' ) )->set_default_value( 'page' )
            ) );

        /* Add Settings for Preview Links */

        $dateTime = current_datetime();
        $begin_date = $dateTime->format('ddmmyyyy');

        $post_types_list = carbon_get_theme_option( 'pm_draft_preview_types' );
        if(!empty($post_types_list)){
            $post_types_array = explode(',', $post_types_list);

            Container::make( 'post_meta', 'Preview Link' )
            ->where( 'post_type', 'IN', $post_types_array  )
            ->set_context( 'side' )->set_priority( 'low' )
            ->add_fields( array(
                Field::make( 'checkbox', 'pw_allow_preview', 'Allow Draft Preview' )
                ->set_attribute( 'data-disabled', 'conditional' ),
                Field::make( 'hidden', 'pm_preview_key', __( '' ) )->set_default_value( $begin_date ),
                Field::make( 'html', 'pm_preview_link_status' )
                ->set_html( 
                    function(){
                        $post_id = $_GET['post'] ?? $_POST['id'] ?? $_POST['post_ID'];                     
                    
                        $current_status = get_post_status($post_id);
                        if($current_status != 'draft'){
                            return '<style>#carbon_fields_container_preview_link { display: none; } </style>'; 
                        }
                    
                    } ),
                Field::make( 'html', 'pm_preview_link' )
                ->set_html( 
                    function(){
                        $post_id = $_GET['post'] ?? $_POST['id'] ?? $_POST['post_ID'];                     
                        if(!empty($post_id) && carbon_get_post_meta( $post_id, 'pw_allow_preview' ) ){
                            $current_status = get_post_status($post_id);
                            if($current_status == 'draft'){
                                $dateTime = current_datetime();
                                $begin_date = $dateTime->format('ddmmyyyy');
                                $preview_key = $begin_date;                            
                                if(!empty(carbon_get_post_meta( $current_id, 'pm_preview_key' ))) $preview_key = carbon_get_post_meta( $current_id, 'pm_preview_key' );
                                $preview_url_key = hash('adler32', $preview_key.$post_id);

                                $cachebust = hash('adler32', $dateTime->format('ddmmyyyyHis'));

                                $link_url_format = '?page_id=';
                                $current_post_type = get_post_type($post_id);
                                if($current_post_type != 'page') $link_url_format = '?post_type='.$current_post_type.'&p=';

                                return '
                                <a class="share-link" href="/'.$link_url_format.$post_id.'&preview=true&pmpreview='.$preview_url_key.'&cb='.$cachebust.'" target="_blank"><span>Share</span><svg xmlns="http://www.w3.org/2000/svg" fill="#d15e00" viewBox="0 0 24 24" width="24" height="24" class="components-menu-items__item-icon has-icon-right" aria-hidden="true" focusable="false"><path d="M19.5 4.5h-7V6h4.44l-5.97 5.97 1.06 1.06L18 7.06v4.44h1.5v-7Zm-13 1a2 2 0 0 0-2 2v10a2 2 0 0 0 2 2h10a2 2 0 0 0 2-2v-3H17v3a.5.5 0 0 1-.5.5h-10a.5.5 0 0 1-.5-.5v-10a.5.5 0 0 1 .5-.5h3V5.5h-3Z"></path></svg></a>
                                <style>.container-carbon_fields_container_preview_link .share-link { display: inline-flex; font-size: 1.2em; font-weight: 500; align-items: center; } .cf-container .cf-field+.cf-field.cf-hidden { border: 0; padding: 2.5px; } .cf-container .cf-field+.cf-field.cf-hidden { border: 0; padding: 2.5px; } .cf-container .cf-field+.cf-field.cf-html { padding: 3px 12px; } .cf-container .cf-field+.cf-field { border-top: 0; }</style>
                                ';
                            } else {
                                return '<style>#carbon_fields_container_preview_link { display: none; } </style>'; 
                                // return '<p>Preview links can\'t be used directly for published posts. You can use the "Rewrite & Republish" feature to create a temporary draft copy, or the "Switch to Draft" option to unpublish the post itself.</p>';
                            }
                        } else {
                            if(!empty($post_id) && $current_status = get_post_status($post_id) == 'draft'){
                                return '
                                <p>Save draft and refresh to generate shareable preview link.</a><style>.cf-container .cf-field+.cf-field.cf-hidden { border: 0; padding: 2.5px; } .cf-container .cf-field+.cf-field.cf-html { padding: 3px 12px; } .cf-container .cf-field+.cf-field { border-top: 0; }</style>
                                ';
                            } else {                            
                                return '<p>Preview links can\'t be used directly for published posts. You can use the <strong>"Rewrite & Republish"</strong> feature to create a temporary draft copy, or the <strong>"Switch to Draft"</strong> option to unpublish the post itself.</p>';
                            }
                        }
                    } )
                ->set_conditional_logic( array(
                    array(
                        'field' => 'pw_allow_preview',
                        'value' => true,
                    )
                ) ),
                Field::make( 'date', 'pw_preview_end', __( 'Preview Expiration' ) )
                ->set_storage_format( 'Ymd' )
                ->set_conditional_logic( array(
                    array(
                        'field' => 'pw_allow_preview',
                        'value' => true,
                    )
                ) ),
                ));
        }
    }
}


/* Add Query Variable for Preview Links */

add_filter('query_vars', 'pm_preview_link_query_var');

function pm_preview_link_query_var($query_vars)
{
    $query_vars[] = 'pmpreview'; // Change it to your desired name
    return $query_vars;
}

add_action('pre_get_posts', 'pm_allow_draft_preview_link');
 
function pm_allow_draft_preview_link($query) {
        /* Check for preview flag in URL, and a non-logged in user status */
        $preview_url_key = get_query_var( 'pmpreview', false);
        if ($preview_url_key && !is_user_logged_in()) {
            /* Check if this post query is the main one */
            if ($query->is_main_query()) {
                /* Check the current post ID for the relevant meta values */
                $current_id = get_query_var( 'page_id');
                if(empty($current_id) && !empty(get_query_var( 'post_type')) ) $current_id = get_query_var( 'p');
                echo '<!-- preview_url: '. $preview_url_key.' -->';
                echo '<!-- current_id: '. $current_id.' -->';
                /* Check whether preview URL is enabled for this post */
                if(function_exists('carbon_get_post_meta')){
                    if(carbon_get_post_meta( $current_id, 'pw_allow_preview' )){
                        /* Get the key value saved for this post, and run it through the same hash algorithm as used in the link, then compare */
                        $preview_key = carbon_get_post_meta( $current_id, 'pm_preview_key' );    
                        $compare_key = hash('adler32', $preview_key.$current_id);
                            echo '<!-- preview_key: '. $preview_key.' -->';
                            echo '<!-- compare_key: '. $compare_key.' -->';
                        if($compare_key == $preview_url_key){
                            /* Check the expiration date and current date */

                            $pw_preview_end = carbon_get_post_meta( $current_id, 'pw_preview_end' );
                            $dateTime = current_datetime();
                            $current_date = $dateTime->format('Ymd');
                            echo '<!-- today: '. $current_date.' -->';
                            echo '<!-- expire: '. $pw_preview_end.' -->';
                            /* If the expiration date is unset, or in the future, show the draft */
                            if(empty($pw_preview_end) || $current_date <= $pw_preview_end) $query->set('post_status', array('publish', 'draft'));
                        }
                    } 
                } else {
                    $current_id = get_query_var( 'page_id');

                    $preview_post = get_post($current_id);

                    if(get_post_status($current_id) == 'draft'){

                        $pm_preview_enabled = get_post_meta( $current_id, 'pm-preview-link-enabled', true );
                    
                        $compare_key = hash('adler32', $preview_post->post_name.$current_id);
                        if($compare_key == $preview_url_key && $pm_preview_enabled){
                            $query->set('post_status', array('publish', 'draft'));
                        }

                    }
                }
            }
        }
}

/* Simplified Non-Carbon-Fields Functionality */

//Register Meta box
add_action( 'add_meta_boxes', function() {
    if ( !function_exists('carbon_get_post_meta') ) {
        add_meta_box( 'pm-preview-link', 'Preview Link', 'pm_preview_link_simple', 'page', 'side' );
    }
} );

//Meta callback function
function pm_preview_link_simple( $post ) {

    $pm_preview_enabled = get_post_meta( $post->ID, 'pm-preview-link-enabled', true );
    
    $check_atts = '';
    $label_style = '';

    if(get_post_status($post->ID) != 'draft'){
        $check_atts .= 'disabled ';
        $label_style = 'style="color:lightgray;" ';
        echo '<p><em>Preview links can\'t be used directly for published posts. You can use the <strong>"Switch to Draft"</strong> option to unpublish the post itself, or if the "Yoast Duplicate Post" plugin is installed, you can use the <strong>"Rewrite & Republish"</strong> feature to create a temporary draft copy.</em></p>';
        $preview_link = '';
    } else {

        $dateTime = current_datetime();
        $cachebust = hash('adler32', $dateTime->format('ddmmyyyyHis'));

        $preview_url_key = hash('adler32', $post->post_name.$post->ID);
        $link_url_format = '?page_id=';
        $preview_link = '<p><a class="share-link" style="display: inline-flex; font-size: 1.2em; font-weight: 500; align-items: center;" href="/'.$link_url_format.$post->ID.'&preview=true&pmpreview='.$preview_url_key.'&cb='.$cachebust.'" target="_blank"><span>Share</span><svg xmlns="http://www.w3.org/2000/svg" fill="#d15e00" viewBox="0 0 24 24" width="24" height="24" class="components-menu-items__item-icon has-icon-right" aria-hidden="true" focusable="false"><path d="M19.5 4.5h-7V6h4.44l-5.97 5.97 1.06 1.06L18 7.06v4.44h1.5v-7Zm-13 1a2 2 0 0 0-2 2v10a2 2 0 0 0 2 2h10a2 2 0 0 0 2-2v-3H17v3a.5.5 0 0 1-.5.5h-10a.5.5 0 0 1-.5-.5v-10a.5.5 0 0 1 .5-.5h3V5.5h-3Z"></path></svg></a></p>';
    }

    if(!empty($pm_preview_enabled)) {
        $check_atts .= 'checked="checked" ';
    }

    ?>
    <label <?php echo $label_style; ?>><input type="checkbox" name="pm-preview-link-enabled" id="pm-preview-link-enabled" value="true" <?php echo $check_atts; ?> >&nbsp;Allow Draft Preview</label>
    <?php echo $preview_link; ?>

    <?php
}

//save meta value with save post hook
add_action( 'save_post', function( $post_id ) {
    if ( isset( $_POST['pm-preview-link-enabled'] ) ) {
        update_post_meta( $post_id, 'pm-preview-link-enabled', $_POST['pm-preview-link-enabled'] );
    } else {
        update_post_meta( $post_id, 'pm-preview-link-enabled', false );
    }
} );

// show meta value after post content
add_filter( 'the_content', function( $content ) {
    $meta_val = get_post_meta( get_the_ID(), 'pm-preview-link-enabled', true );
    if($meta_val) return $content . '<!-- Draft Preview Enabled -->';

    return $content;
} );